from .article.models import *
from .base.models import *
from .gallery.models import *
from .text.models import *
from .uploads.models import *


#import os, sys, pprint
#from importlib import import_module, invalidate_caches
#script_name = os.path.basename(__file__).split('.')[0];
#.= os.path.join(os.path.dirname(os.path.abspath(__file__)), .);
#
#print('—————————————————————— ' + script_name)
#
#for module in os.listdir.:
#    if (module[0] == '_'):
#        continue
#
#    module_path = os.path.join. module)
#
#    if (not os.path.isdir(module_path)):
#        continue
#
#    script_path = os.path.join(module_path, script_name + '.py')
#
#    if (not os.path.isfile(script_path)):
#        continue
#
#    full_name = '.' + module + '.' + script_name
#    mod = import_module(full_name, __package__)
#    print(full_name)
#    print(dir(mod))
#    print('----------')
#
#print('——————————————————————')
#
#print('—asdasdasdadasdasd——————')
#print(dir())

#from .text.models import *

#pprint.pprint(sys.
#pprint.pprint(SimpleText)


#__all__ ==

#import datetime
#
#from django.db import models
#from ckeditor_uploader.fields import RichTextUploadingField
#from ckeditor.fields import RichTextField
#from sorl.thumbnail import ImageField, get_thumbnail


#class SimplePriceLists(KenzoUpload):
#    name = models.CharField(
#        'Название',
##        default = 'noname',
#        max_length = 128,
#    )
#
#    def __str__(self):
#        return self.name
#
#    class Meta:
#        abstract = True
#        verbose_name = 'прайс-лист'
#        verbose_name_plural = 'Прайс-листы'



import re
import datetime
import time
import urllib
import json

from inspect import ismethod

from django.views import generic
from django.utils import timezone
from django.utils.decorators import classonlymethod
from django.db.models import (
    Model, Manager, AutoField, CharField
)
from django.db.models.query import QuerySet
from django.db.models.fields.files import ImageFieldFile
from django.core.exceptions import FieldDoesNotExist
from django.http import (HttpResponse,
    HttpResponseNotFound, HttpResponseForbidden)

from .. import utils

DEFAULT_PAGE_SIZE = 200
DEFAULT_PAGE_SIZE_MAX = 200

class Stream:
    user = None
    query = None
    params = None # from path

    count = None
    skip = None

    keys = [
        ('count', int, DEFAULT_PAGE_SIZE),
        ('skip', int, 0)
    ]

    def __init__(self, HttpRequest, params):
        self.user = HttpRequest.user
        self.query = HttpRequest.GET
        self.params = params
        self.client_params = {}

        for key, type, default in self.keys:
            value = HttpRequest.GET.get(key, default)
            self.client_params[key] = type(value)


def convert_to_json(method):
    def wrapper(self, HttpRequest, **kwargs):
        stream = method(self, HttpRequest, **kwargs)

        if isinstance(stream.data, HttpResponse):
            return stream.data

        if stream.data == None:
            return HttpResponseNotFound()

        response = HttpResponse(
            json.dumps(
                stream.data,
                ensure_ascii = False,
                sort_keys = True
            ),
            content_type = 'application/json'
        )

        if hasattr(stream, 'total'):
            response['Total'] = stream.total

        if hasattr(stream, 'skip'):
            response['Skip'] = stream.skip

        if hasattr(stream, 'count'):
            response['Count'] = stream.count

#        print(stream)

        return response

    return wrapper


class BaseAPI(generic.View):
    model = None
    key = 'id'
#    data = None

    page_size = DEFAULT_PAGE_SIZE
    page_size_max = DEFAULT_PAGE_SIZE_MAX

    @classonlymethod
    def as_api(self, **kwargs):
        self.dynamic_filters = {};
        if len(kwargs) > 0:
            if 'filters' in kwargs:
                # Представление получает новые фильтры из urls.py (.as_api())
                filters = kwargs['filters']
                for key in filters:
#                    print('> > >', key)
                    value = filters[key]
                    if type(value) == tuple and len(value) == 2:
                        self.dynamic_filters.update({
                            key: value
                        })
        return self.as_view()

    available_filters = []
    dynamic_filters = {}
    def get_filters(self, stream):
        filters = {
            'status': True,
        }

        #stream.user.is_staff or stream.user.is_superuser
        try:
            field = self.model._meta.get_field('pub_date')
            filters.update({
                'pub_date__lt': timezone.now()
            })
        except FieldDoesNotExist:
            pass

        if len(self.dynamic_filters) > 0:
#            print('... dynamic_filters:', self.dynamic_filters)
            for key in self.dynamic_filters:
                param_key = self.dynamic_filters[key][0]
                param_class = self.dynamic_filters[key][1]

#                print('param_key', param_key)
#                print('param_class', param_class)

                if param_key in stream.params:
                    value = param_class(stream.params[param_key])
                    filters[key] = value

#        for filter in self.available_filters:
#            if (stream.query.__contains__(filter[0])):
#                self.filters[filter[1]] = stream.query.__getitem__(filter[0])

        return filters


    orders = []
    available_orders = []

    # Данные

    def get_resource(self, HttpRequest, params):
        stream = Stream(HttpRequest, params)

        input_key, field_key = self.get_keys(self.key)

        try:
            query = {
                field_key: stream.params[input_key]
            }

            stream.response = self.model.objects.get(**query)

        except self.model.DoesNotExist:
            stream.response = HttpResponseNotFound()

        return stream


    def get_collection(self, HttpRequest, params):
        stream = Stream(HttpRequest, params)

        stream.response = self.model.objects

        # Проверка и вся хуйня

        # Фильтры
        filters = self.get_filters(stream)
        stream.response = stream.response.filter(**filters)

#        print('... filters', filters)
#        print('..query..', stream.response.query)

        # Сортировка
        if (len(self.orders) > 0):
            stream.response = stream.response.order_by(*self.orders)

        # Пагинация
        stream.total = stream.response.count()
        stream.skip = stream.client_params['skip']
        stream.count = stream.client_params['count']

        if (stream.skip > stream.total):
            stream.skip = stream.total
        elif stream.skip < 0:
            stream.skip = 0

        if stream.count < 0:
            stream.count = 0
        elif stream.count > DEFAULT_PAGE_SIZE_MAX:
            stream.count = DEFAULT_PAGE_SIZE_MAX

        stream.response = stream.response[
            stream.skip : stream.skip + stream.count
        ]

        return stream

    def get_keys(self, string):
#            line = re.search(r'\s-[d]*$', string)
#            line = re.sub(r'\s-[d]*', '', string)
#            print('..', line)

        if ' as ' in string:
            orig, view = string.split(' as ')
        else:
            orig = view = string
        return orig, view

    # NOTE: Префикс используется?
    def export__resource(self, resource, schema = None, prefix = None):
        data = {};
        if type(schema) != tuple:
            schema = self.exported_data

        def get_attr_by_path(resource, path):
            key = path.pop(0)

#            print('..', key, resource)

            if ismethod(resource):
                resource = resource()

            if (resource and hasattr(resource, key)):
                value = getattr(resource, key)
            else:
                value = None
#                print(resource)
#                print(key)

            if len(path) == 0:
                if isinstance(value, datetime.time):
                    return(value.isoformat())

                if isinstance(value, datetime.datetime):
                    return(value.isoformat())

                if ismethod(value):
                    return value()

                return value
            else:
                return get_attr_by_path(value, path)

        for item in schema:
            if type(item) == str:
                orig, view = self.get_keys(item)
                if prefix:
                    orig = prefix + orig

                path = orig.split('.')
                data[view] = get_attr_by_path(resource, path)

            elif type(item) == tuple:
                orig, view = self.get_keys(item[0])
                keys = item[1]
                model = getattr(resource, orig)

                if isinstance(model, Manager):
                    resources = model.all()

                    if type(keys) == tuple:
                        data[view] = self.export(
                            resources,
                            schema = keys,
                        )
                    elif type(keys) == str:
                        data[view] = [ i[keys] for i in self.export(
                            resources,
                            schema = (keys, ),
                        )]
                    else:
                        print('*** Manager: Некорректный тип:', type(keys))

                elif isinstance(model, Model):
                    if type(keys) == str:
                        keys = (keys, )

                    if type(keys) == tuple:
                        data[view] = self.export(
                            model,
                            schema = keys,
                        )
                    else:
                        print('*** Model: Некорректный тип:', type(keys))

                else:
                    print('*** Не Менеджер! ***', manager)

            else:
                print('Что-то непонятное')

        return data

    # Конвертирование данных для выдачи
    def export(self, response, schema = None):
        if isinstance(response, QuerySet):
            if not self.exported_data:
                return None

            data = []

            # FIXME: ЗАПРОСЫ второго уровня НЕ ФИЛЬТРУЮТСЯ

#            filters = {}
#
#            for filter_key in self.__instance['filters']:
#                filter = {filter_key: self.__instance['filters'][filter_key]}
#                try:
#                    field = response.model._meta.get_field(filter_key)
#                    filters.update(filter)
#                except FieldDoesNotExist:
#                    pass
#
##            print('....', filters)
#
#            response = response.filter(**filters)

            for resource in response:
                item = self.export__resource(resource, schema)
                if item != None:
                    data.append(item)

        elif isinstance(response, Model):
            if not self.exported_data:
                return None

            data = self.export__resource(response, schema);
        else:
            return None

        return data

    def get_exported_resource(self, HttpRequest, params):
        stream = self.get_resource(HttpRequest, params)
        stream.data = self.export(stream.response)
        return stream

    def get_exported_collection(self, HttpRequest, params):
        stream = self.get_collection(HttpRequest, params)
        stream.data = self.export(stream.response)
        return stream

class ResourceMixin:
#     def __init__(self):
#         self.test = True

    '''Заголовок ресурса'''
    def head(self, HttpRequest, **kwargs):
        pass

    '''Ресурс'''
    @convert_to_json
    def get(self, HttpRequest, **kwargs):
        stream = self.get_exported_resource(HttpRequest, kwargs)
        return stream

    '''Изменение ресурса'''
    def post(self, HttpRequest, *args, **kwargs):
        pass

    '''Удаление ресурса'''
    def delete(self, HttpRequest, **kwargs):
        pass


class CollectionMixin:
    '''Заголовок коллекции'''
    def head(self, HttpRequest, **kwargs):
        pass

    '''Коллекция'''
    @convert_to_json
    def get(self, HttpRequest, **kwargs):
        stream = self.get_exported_collection(HttpRequest, kwargs)
        return stream

    '''Новый ресурс в коллекции'''
    def post(self, HttpRequest, *args, **kwargs):
        pass



import datetime
from django.db import models
from ckeditor_uploader.fields import RichTextUploadingField

from ..base import models as kk

class BasicArticle(kk.Base):
    headline = models.CharField(
        'Заголовок',
        max_length = 255,
#        help_text="Текст: <em>Текст</em>."
    )

    content = RichTextUploadingField(
        'Содержимое',
        blank = True
    )

    pub_date = models.DateTimeField(
        'Время публикации',
        default = datetime.datetime.now
    )

    def __str__(self):
        return self.headline

    class Meta:
        abstract = True
        ordering = ['-pub_date']
        verbose_name = 'статья'
        verbose_name_plural = 'Статьи'


class ArticleAnnotationMixin(models.Model):
    annotation = models.TextField(
        'Аннотация',
        help_text = 'Заполните аннотацию и в списке статей<br />\
            будет показан этот текст, вместо основного текста статьи',
        blank = True
    )

    class Meta:
        abstract = True


class ArticleCoverMixin(models.Model):
    cover = models.ImageField(
        'Обложка',
        upload_to = 'images',
        blank = True
    )

    class Meta:
        abstract = True



from django.db import models

class Base(models.Model):
    created_date = models.DateTimeField(
        auto_now_add = True
    )

    modified_date = models.DateTimeField(
        auto_now = True
    )

    status = models.BooleanField(
        'Опубликовано',
        default = True
    )

    class Meta:
        abstract = True


class OrderedMixin(models.Model):
    position = models.PositiveSmallIntegerField(
        'Позиция',
        null = True,
        blank = True
    )

    def unordered(self):
        return self.position == None

    class Meta:
        abstract = True


class Ordered(OrderedMixin, Base):
    class Meta:
        abstract = True


class XMLCodeMixin(models.Model):
    xml = models.TextField(
        'Код',
        blank = True
    )

    class Meta:
        abstract = True



from django.contrib import admin

class SimpleTextAdmin(admin.ModelAdmin):
    list_display = ['codename', 'status', 'note']
    list_per_page = 80
    ordering = ['codename']
    fields = ['status', 'codename', 'note', 'content']



from django.db import models
from ckeditor_uploader.fields import RichTextUploadingField
#from ckeditor.fields import RichTextField

from ..base import models as base

class SimpleText(base.Base):
    codename = models.CharField(
        'Кодовое имя',
        help_text="<em>Латинскими буквами</em>",
        max_length = 128,
        unique = True
    )

    note = models.CharField(
        'Заметка',
#        default = 'noname',
        max_length = 128,
        null = True,
        blank = True
    )

    content = RichTextUploadingField(
        'Содержимое',
        blank=True
    )

    def __str__(self):
        return self.codename

    class Meta:
        abstract = True
        verbose_name = 'Текстовый блок'
        verbose_name_plural = 'Текстовые блоки'



import os, json
from django.http import (HttpResponse,
    HttpResponseNotFound, HttpResponseForbidden)


def get_modules():
    path = os.path.dirname(
        os.path.abspath(__file__)
    )

    names = os.listdir(path = path)

    return list(filter(
        lambda name:
            os.path.isdir(os.path.join(path, name)) and '__' not in name,
        names
    ))


def json_response(data):
    return HttpResponse(
        json.dumps(data, ensure_ascii = False, sort_keys = True),
        content_type = 'application/json')


def make_vector(array, key):
    output = {}

    for item in array:
        output[item[key]] = item

    return output


def get_by_id(the_list, id):
    for item in the_list:
        if item['id'] == id:
            return item

    return None


def access_check(method):
    def wrapper(self, HttpRequest, **kwargs):
        user = HttpRequest.user

        # Анонимные запросы мимо
        if user.id == None:
            return HttpResponseForbidden()

        # Ограничение доступа к информации пользователям
        # с незаполненными профилями
        if not user.profile.is_completed:
            if not (user.is_staff or user.is_superuser):
                return HttpResponseForbidden()

        # Из Lazy в нормальный объект.
        HttpRequest.user = user

        return method(self, HttpRequest, **kwargs)

    return wrapper


def export_to_json(method):
    def wrapper(self, HttpRequest, **kwargs):
        data = method(self, HttpRequest, **kwargs)

        if isinstance(data, HttpResponse):
            return data

        if data == None:
            return HttpResponseNotFound()

        return json_response(data)

    return wrapper



from django.db import models

from sorl.thumbnail.fields import ImageField
from sorl.thumbnail import get_thumbnail

from ..base import models as base


class GalleryBase(base.Ordered):
    original_image = models.ImageField(
        'Изображение',
        upload_to = 'images'
    )

    def thumb(self):
        return get_thumbnail(self.image, '80x60', crop = 'center')

    class Meta:
        abstract = True


class SimpleGallery(GalleryBase):

    caption = models.TextField(
        'Описание',
        null = True,
        blank = True
    )

    class Meta:
        abstract = True
        verbose_name = 'слайд'
        verbose_name_plural = 'галерея'


class SimpleBanner(GalleryBase):
    text = models.CharField(
        'текст',
        help_text="<em>Максимум 140 знаков</em>",
        max_length = 140,
        null = True,
        blank = True
    )

    class Meta:
        abstract = True
        verbose_name = 'баннер'
        verbose_name_plural = 'баннеры'



from django.contrib import admin

class SimpleUploadAdmin(admin.ModelAdmin):
    list_display = ['codename', 'status', 'file']
    list_per_page = 80
    ordering = ['codename']

    fields = ['status', 'codename', 'file']



from django.contrib import admin
from django.db import models
from django import forms
from django.forms import ClearableFileInput
from django.utils.safestring import mark_safe
from django.utils.html import format_html

from sorl.thumbnail.fields import ImageField
from sorl.thumbnail import get_thumbnail


class ImageWidget(ClearableFileInput):
    template_name = 'admin/widgets/image.html'

#    def render(self, name, value, attrs = None, renderer = None):
##        get_thumbnail(obj.image, '150x150')
##        print(type(value));
##        print(attrs);
#        output = super(ImageWidget, self).render(name, value, attrs, renderer)
#        return output


class ImageListMixin(admin.ModelAdmin):

    def image_list_display(self, obj, **kwargs):
        return format_html('<img src="{url}" />'.format(
            url = get_thumbnail(obj.original_image, '150x150').url
        ))
    image_list_display.short_description = 'Изображение'


class GalleryAdminWidget(forms.FileInput):
#    template_with_initial = '%(clear_template)s<br />%(input_text)s: %(input)s'
#    template_with_clear = ('%(clear)s <label style="width:auto" for="%(clear_checkbox_id)s">'
#                           '%(clear_checkbox_label)s</label>')

    def render(self, name, value, attrs=None):
        output = super(SimpleGalleryAdminWidget, self).render(name, value, attrs)
        if value and hasattr(value, 'url'):
            try:
                thumb = get_thumbnail(value, '150')
            except Exception as e:
                pass
                logger.warn('Не удаётся получить миниатюры', exc_info=e)

            try:
                output = (
                    '<div class="kk-thumbnail">'
                    '<a class="kk-thumbnail__link" href="%s" target="_blank">'
                    '<img class="kk-thumbnail__image" src="%s"></a>'
                    '<br />%s'
                    '</div>'
                ) % (value.url, thumb.url, output)
            except AttributeError:
                pass
        return mark_safe(output)


class GalleryBaseAdmin(admin.ModelAdmin):
    list_display = ('image_list_display', 'position', 'status')
    list_per_page = 80
#    ordering = ('-is_unordered', 'position', )

    fields = ('status', 'position', 'image',)

#    def get_queryset(self, request):
#        qs = super(GalleryBaseAdmin, self).get_queryset(request)
#        qs = qs.annotate(is_unordered = models.Count('position'))
#        return qs
#
#    def is_unordered(self, obj):
#        return obj.is_unordered
#    is_unordered.admin_order_field = 'is_unordered'


    def image_list_display(self, obj, **kwargs):
        return format_html('<img src="%s" />' % get_thumbnail(obj.image, '150x150').url)
    image_list_display.short_description = 'Изображение'

    def formfield_for_dbfield(self, obj, **kwargs):
        if isinstance(obj, ImageField):
            return obj.formfield(widget = GalleryAdminWidget)
        sup = super(GalleryBaseAdmin, self)
        return sup.formfield_for_dbfield(obj, **kwargs)

    formfield_overrides = {
        models.TextField: {'widget': forms.Textarea(attrs = {'rows': 4})},
    }


class SimpleGalleryAdmin(GalleryBaseAdmin):
    list_display = ('image_list_display', 'caption', 'position', 'status')
    fields = ('status', 'position', 'caption', 'image')

class SimpleBannerAdmin(GalleryBaseAdmin):
    list_display = ('image_list_display', 'text', 'position', 'status')
    fields = ('status', 'position', 'text', 'image')



import datetime
from django.db import models

from ..base import models as base


class UploadBase(base.Ordered):
    upload_date = models.DateTimeField(
        'Время загрузки',
        default = datetime.datetime.now
    )

    class Meta:
        abstract = True
        ordering = ['-upload_date']



class Upload(UploadBase):
    file = models.FileField(
        'Файл',
        upload_to = 'uploads'
    )

    class Meta:
        abstract = True



class SimpleUpload(Upload):
    codename = models.CharField(
        'Кодовое имя',
        help_text="<em>Латинскими буквами</em>",
#        default = 'noname',
        max_length = 128,
        unique = True
    )

    def __str__(self):
        return self.codename

    class Meta:
        abstract = True
        verbose_name = 'файл'
        verbose_name_plural = 'Файлы'



import os
import kk
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

CLASSIFIERS = [
#    Development Status :: 1 - Planning
#    Development Status :: 2 - Pre-Alpha
#    Development Status :: 3 - Alpha
#    Development Status :: 4 - Beta
#    Development Status :: 5 - Production/Stable
#    Development Status :: 6 - Mature
#    Development Status :: 7 - Inactive
    'Development Status :: 3 - Alpha',

#    Environment :: Console
#    Environment :: Console :: Curses
#    Environment :: Console :: Framebuffer
#    Environment :: Console :: Newt
#    Environment :: Console :: svgalib
#    Environment :: Handhelds/PDA's
#    Environment :: MacOS X
#    Environment :: MacOS X :: Aqua
#    Environment :: MacOS X :: Carbon
#    Environment :: MacOS X :: Cocoa
#    Environment :: No Input/Output (Daemon)
#    Environment :: OpenStack
#    Environment :: Other Environment
#    Environment :: Plugins
#    Environment :: Web Environment
#    Environment :: Web Environment :: Buffet
#    Environment :: Web Environment :: Mozilla
#    Environment :: Web Environment :: ToscaWidgets
#    Environment :: Win32 (MS Windows)
#    Environment :: X11 Applications
#    Environment :: X11 Applications :: Gnome
#    Environment :: X11 Applications :: GTK
#    Environment :: X11 Applications :: KDE
#    Environment :: X11 Applications :: Qt
    'Environment :: Web Environment',

#    Framework :: Flask
    'Framework :: Django',
    'Framework :: Django :: 2.0',

    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',

#    'Natural Language :: English',
    'Natural Language :: Russian',


    'Operating System :: OS Independent',

    'Programming Language :: Python',
    'Programming Language :: Python :: 3 :: Only',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',

    'Topic :: Software Development :: Libraries',
]

INSTALL_REQUIREMENTS = [
    'Django>=2.0,<3.0',
    'django-ckeditor>=5.3,<6',
    'Pillow>=5',
    'PyYAML>=3.12,<4',
    #'Markdown2>=2.3,<3',
    'sorl-thumbnail>=12.4.1,<13'
]

setup(
    name = 'django-kk',
    version = kk.__version__,

    license = 'MIT',
    description = 'Kenzo Kit for Django.',
    long_description = README,

    author = 'icw82',
    author_email = 'icw82@yandex.ru',
    url = 'https://icw82.ru/',

    packages = find_packages(),
    include_package_data = True,

    classifiers = CLASSIFIERS,
    keywords = 'REST API JSON',
    install_requires = INSTALL_REQUIREMENTS,
)



from django.contrib import admin
from django.db import models
from django import forms
from django.utils.safestring import mark_safe
from django.utils.html import format_html

class BaseAdmin(admin.ModelAdmin):
    list_per_page = 80

class OrderedAdmin(BaseAdmin):
    list_display = ('position', 'status')
#    ordering = ('-is_unordered', 'position', )

    fields = ('status', 'position', )

    def get_queryset(self, request):
        qs = super(OrderedAdmin, self).get_queryset(request)
        qs = qs.annotate(is_unordered = models.Count('position'))
        return qs

    def is_unordered(self, obj):
        return obj.is_unordered
    is_unordered.admin_order_field = 'is_unordered'


class InlinesBeforeCreatingObjectAdminMixin(admin.ModelAdmin):

    def get_inline_instances(self, request, obj = None):
        if obj:
            return [
                inline(self.model, self.admin_site) for inline in self.inlines
            ]
        else:
            return []
